#include "StdAfx.h"
#include "WarMmcSnapin.h"
#include "WmsNodeAuthModules.h"   // class implemented
#include "WarCollector.h"
#include "WarIfServer.h"
#include "WarIfAuthModule.h"
#include "WmsNodeAuthModule.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////


// {27BF7AA6-1CA1-43ae-84CD-3347E6E834D6}
const GUID WmsNodeAuthModules::thisGuid = 
{ 0x27bf7aa6, 0x1ca1, 0x43ae, { 0x84, 0xcd, 0x33, 0x47, 0xe6, 0xe8, 0x34, 0xd6 } };


WmsNodeAuthModules::WmsNodeAuthModules(war_if_site_ptr_t& rSitePtr)
: mSitePtr(rSitePtr) // Deleted by parent
{
}

WmsNodeAuthModules::~WmsNodeAuthModules()
{
}

//============================= LIFECYCLE ====================================
//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
//============================= CALLBACK     ===================================

HRESULT WmsNodeAuthModules::OnExpand(IConsoleNameSpace *pConsoleNameSpace, 
                                 IConsole *pConsole, 
                                 HSCOPEITEM parent)
{
    if (GetHandle() == NULL) 
        SetHandle((HANDLE)parent);

    if (!bExpanded) 
    {
        war_if_aut_module_list_t mod_list;
        mSitePtr->EnumAuthModules(mod_list);

        for(war_if_aut_module_list_t::iterator P = mod_list.begin()
            ; P != mod_list.end()
            ; ++P)
        {
            CDelegationBase *pnew_node;
            SCOPEDATAITEM sdi;

            pnew_node = new WmsNodeAuthModule(mSitePtr, *P);

            ZeroMemory(&sdi, sizeof(SCOPEDATAITEM));
            sdi.mask = SDI_STR|   // Displayname is valid
                SDI_PARAM     |   // lParam is valid
                SDI_IMAGE     |   // nImage is valid
                SDI_OPENIMAGE |   // nOpenImage is valid
                SDI_PARENT;
            
            sdi.relativeID  = (HSCOPEITEM)parent;
            sdi.nImage      = pnew_node->GetBitmapIndex();
            sdi.nOpenImage  = pnew_node->GetBitmapIndex();
            sdi.displayname = MMC_CALLBACK;
            sdi.lParam      = (LPARAM)pnew_node;     // The cookie
            
            HRESULT hr = pConsoleNameSpace->InsertItem(&sdi);
            _ASSERT( SUCCEEDED(hr) );
            
            pnew_node->SetHandle((HANDLE)sdi.ID);
        }
    }
 
    return S_OK;   
}

HRESULT WmsNodeAuthModules::OnShow(IConsole *pConsole, 
                                  BOOL bShow, 
                                  HSCOPEITEM scopeitem)
{
    AFX_MANAGE_STATE(AfxGetStaticModuleState());

    HRESULT      hr = S_OK;
    
    IHeaderCtrl *pHeaderCtrl = NULL;
    IResultData *pResultData = NULL;
    
    if (bShow) 
    {
        hr = pConsole->QueryInterface(IID_IHeaderCtrl, (void **)&pHeaderCtrl);
        _ASSERT(SUCCEEDED(hr));
        
        hr = pConsole->QueryInterface(IID_IResultData, (void **)&pResultData);
        _ASSERT(SUCCEEDED(hr));

        CString hdr1_name, hdr2_name;
        hdr1_name.LoadString(IDS_HDRNAME);
        hdr2_name.LoadString(IDS_HDRAUTTYPE);

        MAKE_WIDEPTR_FROMTSTR(ptr_hdr1, hdr1_name);
        MAKE_WIDEPTR_FROMTSTR(ptr_hdr2, hdr2_name);
        
        // Set the column headers in the results pane
        hr = pHeaderCtrl->InsertColumn( 0, ptr_hdr1, 0, MMCLV_AUTO );
        _ASSERT( S_OK == hr );
        hr = pHeaderCtrl->InsertColumn( 1, ptr_hdr2, 0, MMCLV_AUTO );
        _ASSERT( S_OK == hr );
                
        hr = pResultData->DeleteAllRsltItems();
        _ASSERT( SUCCEEDED(hr) );
        
        pHeaderCtrl->Release();
        pResultData->Release();
    }
    
    return hr;
}


//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================

const int WmsNodeAuthModules::GetBitmapIndex()
{
    return NICON_FOLDER;
}

const _TCHAR *WmsNodeAuthModules::GetDisplayName(int nCol)
{
    static _TCHAR szDisplayName[256] = {0};
    
    if (!*szDisplayName)
    {
        LoadString(g_hinst, 
            IDS_NODE_AUTH_MODULES, 
            szDisplayName, 
            sizeof(szDisplayName));
    }
    
    return szDisplayName; 
}

const GUID & WmsNodeAuthModules::getNodeType()
{
    return thisGuid; 
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

