#include "StdAfx.h"
#include "WarMmcSnapin.h"
#include "WmsNodeUsers.h"   // class implemented
#include "WarCollector.h"
#include "WarIfUser.h"
#include "WmsNodeUser.h"


/////////////////////////////// PUBLIC ///////////////////////////////////////


// {22C848E8-ECD9-4604-8A80-195A5ABB9683}
const GUID WmsNodeUsers::thisGuid = 
{ 0x22c848e8, 0xecd9, 0x4604, { 0x8a, 0x80, 0x19, 0x5a, 0x5a, 0xbb, 0x96, 0x83 } };


WmsNodeUsers::WmsNodeUsers(war_if_site_ptr_t& rSitePtr)
: mSitePtr(rSitePtr) // Deleted by parent
{
}

WmsNodeUsers::~WmsNodeUsers()
{
}

//============================= LIFECYCLE ====================================
//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================
//============================= CALLBACK   ===================================

HRESULT WmsNodeUsers::OnExpand(IConsoleNameSpace *pConsoleNameSpace, 
                                 IConsole *pConsole, 
                                 HSCOPEITEM parent)
{
    if (GetHandle() == NULL) 
        SetHandle((HANDLE)parent);

    if (!bExpanded) 
    {
        // Enumerate all the users from all the active auth-modules
        war_if_user_set_t user_set;

        mSitePtr->EnumAllUsers(user_set);
       
        for(war_if_user_set_t::iterator P = user_set.begin()
            ; P != user_set.end()
            ; ++P)

        {
            CDelegationBase *pnew_node;
            SCOPEDATAITEM sdi;

            pnew_node = new WmsNodeUser(*P);

            ZeroMemory(&sdi, sizeof(SCOPEDATAITEM));
            sdi.mask = SDI_STR|   // Displayname is valid
                SDI_PARAM     |   // lParam is valid
                SDI_IMAGE     |   // nImage is valid
                SDI_OPENIMAGE |   // nOpenImage is valid
                SDI_PARENT    |
                SDI_CHILDREN;
            
            sdi.relativeID  = (HSCOPEITEM)parent;
            sdi.nImage      = pnew_node->GetBitmapIndex();
            sdi.nOpenImage  = pnew_node->GetBitmapIndex();
            sdi.displayname = MMC_CALLBACK;
            sdi.lParam      = (LPARAM)pnew_node;     // The cookie
            
            HRESULT hr = pConsoleNameSpace->InsertItem(&sdi);
            _ASSERT( SUCCEEDED(hr) );
            
            pnew_node->SetHandle((HANDLE)sdi.ID);
        }
    }
 
    return S_OK;   
}

HRESULT WmsNodeUsers::OnShow(IConsole *pConsole, 
                             BOOL bShow, 
                             HSCOPEITEM scopeitem)
{
    AFX_MANAGE_STATE(AfxGetStaticModuleState());

    HRESULT      hr = S_OK;
    
    IHeaderCtrl *pHeaderCtrl = NULL;
    IResultData *pResultData = NULL;
    
    if (bShow) 
    {
        hr = pConsole->QueryInterface(IID_IHeaderCtrl, (void **)&pHeaderCtrl);
        _ASSERT(SUCCEEDED(hr));
        
        hr = pConsole->QueryInterface(IID_IResultData, (void **)&pResultData);
        _ASSERT(SUCCEEDED(hr));

        CString hdr1_name, hdr2_name;
        hdr1_name.LoadString(IDS_HDRNAME);
        hdr2_name.LoadString(IDS_HDRAUTTYPE);

        MAKE_WIDEPTR_FROMTSTR(ptr_hdr1, hdr1_name);
        MAKE_WIDEPTR_FROMTSTR(ptr_hdr2, hdr2_name);
        
        // Set the column headers in the results pane
        hr = pHeaderCtrl->InsertColumn( 0, ptr_hdr1, 0, MMCLV_AUTO );
        _ASSERT( S_OK == hr );
        hr = pHeaderCtrl->InsertColumn( 1, ptr_hdr2, 0, MMCLV_AUTO );
        _ASSERT( S_OK == hr );
                
        hr = pResultData->DeleteAllRsltItems();
        _ASSERT( SUCCEEDED(hr) );
        
        pHeaderCtrl->Release();
        pResultData->Release();
    }
    
    return hr;
}



//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================

const int WmsNodeUsers::GetBitmapIndex()
{
    return NICON_USER_FOLDER;
}

const _TCHAR *WmsNodeUsers::GetDisplayName(int nCol)
{
    static _TCHAR szDisplayName[256] = {0};
    
    if (!*szDisplayName)
    {
        LoadString(g_hinst, 
            IDS_NODE_USERS, 
            szDisplayName, 
            sizeof(szDisplayName));
    }
    
    return szDisplayName; 
}

const GUID & WmsNodeUsers::getNodeType()
{
    return thisGuid; 
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

