#include "StdAfx.h"
#include "WarOptionList.h"   // class implemented

#ifndef WAR_LOG
#   include "WarLog.h"
#endif

/////////////////////////////// PUBLIC ///////////////////////////////////////

WarOptionList *WarOptionList::mspGlobalOptions;

//============================= LIFECYCLE ====================================


WarOptionList::WarOptionList(void)
{
}

WarOptionList::WarOptionList(const WarOptionList& from)
: type_t(from)
{
}

//============================= OPERATORS ====================================

WarOptionList& WarOptionList::operator = (const WarOptionList& from)
{
    type_t::operator = (from);
    return *this;
}


WarOptionList& WarOptionList::operator += (const WarOptionList& from)
{
    for(const_iterator P = from.begin()
        ; P != from.end()
        ; P++)
    {
        insert((*P));
    }
    return *this;
}

//============================= OPERATIONS ===================================

const war_option_ptr_t WarOptionList::GetOption(const std::string& name) const
{
    war_option_ptr_t srch = new WarOption(name.c_str());
    const_iterator P = find(srch);
    if (P != end())
        return (*P);

    WarLog err_log(WARLOG_ERROR, "WarOptionList::GetOption()");

    err_log << "Could not find option: \"" 
        << name
        << "\"."
        << war_endl;
    
    WarThrow(WarError(WAR_ERR_OBJECT_NOT_FOUND), NULL);
}



WarOptionList& WarOptionList::GetGlobalOptions()
{
    if (!mspGlobalOptions)
    {
        mspGlobalOptions = new WarOptionList;
    }
    
    return *mspGlobalOptions;
}

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================

bool WarOptionList::HaveOption(war_ccstr_t name)
{
    WarOption srch(name);
    const_iterator P = find(&srch);
    return P != end();
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

