#include "StdAfx.h"
#include "WarSvrProtocolFtpDataConn.h"   // class implemented

#ifndef WAR_AUTO_LOCK_H
#   include "WarAutoLock.h"
#endif

#ifndef WAR_SVR_PROTOCOL_FTP_H
#   include "WarSvrProtocolFtp.h"
#endif

#ifndef WAR_FILE_DRIVER_FILE_H
#   include "WarFileDriverFile.h"
#endif

#ifndef WAR_LOG_H
#   include "WarLog.h"
#endif

#ifndef WAR_PERFMON_DEF_H
#   include "WarPerfmonDef.h"
#endif

#ifndef WAR_FUNCTION_PROFILER_H
#   include "WarFunctionProfiler.h"
#endif

#define AUTO_LOCK \
        war_datasck_ptr_t my_ref_ptr((WarSvrProtocolFtpDataConn *)this); \
        WarAutoLock my_lock((WarCriticalSection &)mrServer.mSvrLock);

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================

WarSvrProtocolFtpDataConn::WarSvrProtocolFtpDataConn(
    war_socket_io_ptr_t& companionPtr,
    WarSvrProtocolFtp& rServer)
: WarSocket(companionPtr),
mrServer(rServer),
mDirection(D_INVALID),
mHaveSeenEof(false),
mPendingIoRequests(0),
mIoSerialLast(0),
mIoSerialFirst(0),
mHasFinished(false),
mUseFileCallback(false),
mIsInitialized(false),
mTimeOutSeconds(120)
{
    WarLog debug_log(WARLOG_DEBUG, 
        "WarSvrProtocolFtpDataConn::WarSvrProtocolFtpDataConn()");

    WAR_DB_PERFMON_INC(WAR_PRFDEBUG_NUM_FTPDATA);
    
    if (debug_log)
    {
        debug_log << "Creating FTP data socket "
            << GetSeqNumber()
            << war_endl;
    }
}// WarSvrProtocolFtpDataConn


WarSvrProtocolFtpDataConn::~WarSvrProtocolFtpDataConn()
{
    WarLog debug_log(WARLOG_DEBUG, 
        "WarSvrProtocolFtpDataConn::~WarSvrProtocolFtpDataConn()");

    if (debug_log)
    {
        debug_log << "Destroying FTP data socket "
            << GetSeqNumber()
            << war_endl;
    }

    WAR_DB_PERFMON_DEC(WAR_PRFDEBUG_NUM_FTPDATA);

}// ~WarSvrProtocolFtpDataConn


//============================= OPERATORS ====================================



//============================= OPERATIONS ===================================


void WarSvrProtocolFtpDataConn::SendFile(war_svrfile_ptr_t& filePtr)
{
    mDirection = D_SENDING;

    int cps = mrServer.GetIntOption("ftpd_CPSOUTMAXCPS");
    if (cps > 0)
    {
        GetPlugins(WSPE_PRE_ON_SENT).push_back(
            new WarSocketPluginPreOnSent(this, 
                mrServer.GetIntOption("ftpd_CPSOUTSAMPLEDELAY"),
                mrServer.GetIntOption("ftpd_CPSOUTHOLDDELAY"),
                mrServer.GetIntOption("ftpd_CPSOUTMINCNT"),
                mrServer.GetIntOption("ftpd_CPSOUTMAXCNT"),
                mrServer.GetIntOption("ftpd_CPSOUTMINCPS"),
                cps));
    }

    StartTransfer(filePtr);
}

void WarSvrProtocolFtpDataConn::ReceiveFile(war_svrfile_ptr_t& filePtr)
{
    mDirection = D_RECEIVING;

    int cps = mrServer.GetIntOption("ftpd_CPSINMAXCPS");
    if (cps > 0)
    {
        GetPlugins(WSPE_PRE_ON_SENT).push_back(
            new WarSocketPluginPreOnReceived(this, 
                mrServer.GetIntOption("ftpd_CPSINSAMPLEDELAY"),
                mrServer.GetIntOption("ftpd_CPSINHOLDDELAY"),
                mrServer.GetIntOption("ftpd_CPSINMINCNT"),
                mrServer.GetIntOption("ftpd_CPSINMAXCNT"),
                mrServer.GetIntOption("ftpd_CPSINMINCPS"),
                cps));
    }
    
    StartTransfer(filePtr);
}

void WarSvrProtocolFtpDataConn::Abort(const war_error_definitions errReason)
{
    OnTransferDone(WarError(errReason));
}

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================

// called by the control socket
bool WarSvrProtocolFtpDataConn::IsTimeOut() const
{
    return GetIdleTimeSince().IsTimeOut(mTimeOutSeconds);
}

/////////////////////////////// PROTECTED  ///////////////////////////////////

void WarSvrProtocolFtpDataConn::StartTransfer(war_svrfile_ptr_t& filePtr)
{
    WarLog debug_log(WARLOG_DEBUG, 
        "WarSvrProtocolFtpDataConn::StartTransfer()");

    if (debug_log)
    {
        debug_log << "Starting transfer on FTP data socket "
            << GetSeqNumber()
            << " "
            << filePtr->GetUrl()
            << war_endl;
    }

    WAR_PERFMON_INC(WAR_PRFSVR_TRANSFERS_NOW)

    // Allocate transfer buffers
    {
        mNumBufSegments = mrServer.GetIntOption("ftpd_DATABUFSEGMENTS"); 
        if (mNumBufSegments < 1)
            mNumBufSegments = 1;

        mNumBuffers = mrServer.GetIntOption("ftpd_DATABUFCNT");
        if (mNumBuffers < 1)
            mNumBuffers = 1;
        
        mBufferLen = mrServer.GetIntOption("ftpd_DATABUFSIZE");
        if (mBufferLen < 512)
            mBufferLen = 512;

        filePtr->GetFileDriverFile().OnFtpBufferSize(
            mNumBuffers,
            mBufferLen,
            mNumBufSegments);

        mSegmentLen = mBufferLen / mNumBufSegments;
        
        for(int i = 0; i < mNumBuffers; i++)
            mFreeQueue.push_back(
                new WarTransferBuffer(mBufferLen, this, i));
        
        mFilePtr = filePtr;
        if (mFilePtr->GetFlags() & WarFileEnums::F_CALLBACK)
        {
            mFilePtr->AssignCallback(CallbackFunc, this);
            mUseFileCallback = true;
        }

        mIsInitialized = true;
        
        if (mpCompanion->GetCurrentState() != WAR_SCKSTATE_CONNECTED)
            return;
    }

    mTimeOutSeconds = mrServer.GetIntOption("ftpd_DATATIMEOUT");

    PumpData();
}

void WarSvrProtocolFtpDataConn::KillBuffers()
{
    while(!mFreeQueue.empty())
    {
        //const WarTransferBuffer &rTb = mFreeQueue.front();
        mFreeQueue.pop_front();
        mNumBuffers--;
    }

    while(!mPendingInQueue.empty())
    {
        //const WarTransferBuffer &rTb = mFreeQueue.front();
        mPendingInQueue.erase(mPendingInQueue.begin());
        mNumBuffers--;
    }

    WarLog debug_log(WARLOG_DEBUG, "WarSvrProtocolFtpDataConn::KillBuffers()");

    if (debug_log)
    {
        debug_log << mNumBuffers << " buffers and " 
            << mPendingIoRequests << " IO requests left after kill."
            << war_endl;
    }
}

void WarSvrProtocolFtpDataConn::PumpData()
{
    WAR_PROFILE_FUNC(WAR_FP_FTPDATA_PUMP);

    if (!mIsInitialized)
        return;

#ifdef DEBUG
    assert (mNumBuffers == (mPendingIoRequests 
        + mPendingInQueue.size()
        + mFreeQueue.size()));
#endif

    while(!mFreeQueue.empty())
    {
        if (D_SENDING == mDirection)
        {
            // Send data
            try
            {
                WarTransferBuffer *pbuf = &(*mFreeQueue.front());
                if (mUseFileCallback)
                {
                    WAR_PROFILE_FUNC(WAR_FP_FTPDATA_PUMP_FREAD_ASYNCH);

                    pbuf->mSerialNum = mIoSerialLast++;
                    mFilePtr->ReadWithCallback(mFreeQueue.front());
                    ++mPendingIoRequests;
                }
                else
                {
                    WAR_PROFILE_FUNC(WAR_FP_FTPDATA_PUMP_FREAD_SYNCH);
                    pbuf->mBytesUsed = mFilePtr->Read(
                        pbuf->mBuffer, 
                        pbuf->GetLength());

                    if (0 == pbuf->mBytesUsed)
                    {
                        WAR_PROFILE_FUNC(WAR_FP_FTPDATA_PUMP_FREAD_CHKEOF);
                        if (mFilePtr->IsEof())
                            WarThrow(WarError(WAR_FERR_END_OF_FILE), NULL);
                        else
                        {
                            // No data fetched, - not callback-mode
                            // and no error. 
                            // We must handle this as a error-situation
                            WarLog err_log(WARLOG_ERROR, "WarSvrProtocolFtpDataConn::PumpData()");
                            
                            err_log << "Read() from file returen zero bytes in non-callback mode. No error was reported. This should never happen!. Aborting transfer."
                                << war_endl;
                            
                            WarThrow(WarError(WAR_ERR_SYSTEM_ERROR), NULL);
                        }
                    }
                    {
                        WAR_PROFILE_FUNC(WAR_FP_FTPDATA_PUMP_SCK_OUT);
                        SendWithCallback(mFreeQueue.front(), mNumBufSegments);
                        ++mPendingIoRequests;
                    }
                }
            }
            catch(WarException& ex)
            {
                if (ex.LocalError() == WAR_FERR_END_OF_FILE)
                {
                    mHaveSeenEof = true;
                    KillBuffers();
                    continue;
                }
                
                // Abort
                WarLog warn_log(WARLOG_WARNINGS, "WarSvrProtocolFtpDataConn::PumpData()");
                warn_log << "Caught unexpected exception "
                    << ex.Explain()
                    << " Aborting transfer"
                    << war_endl;
                OnTransferDone(ex);
                return;
            }
        }
        else
        {
            WAR_PROFILE_FUNC(WAR_FP_FTPDATA_PUMP_RECEIVING);
            // Receive data
            RecvWithCallback(mFreeQueue.front(), mNumBufSegments);
            ++mPendingIoRequests;
        }
        
        mFreeQueue.pop_front();
    }
    
    {
        WAR_PROFILE_FUNC(WAR_FP_FTPDATA_PUMP_SENDLOOP);

        while(!mPendingInQueue.empty()
            && ((*mPendingInQueue.begin())->mSerialNum 
            == mIoSerialFirst))
        {
            ++mIoSerialFirst;
            if (D_SENDING == mDirection)
            {
                SendWithCallback((*mPendingInQueue.begin()), mNumBufSegments);
                ++mPendingIoRequests;
                
                mPendingInQueue.erase(mPendingInQueue.begin());
            }
            else
            {
                assert(false); // only relevant on sending!
            }
        }
    }
    
#if 0
    {
        WarLog debug_log(WARLOG_DEBUG, "WarSvrProtocolFtpDataConn::PumpData()");
        
        debug_log << "mHaveSeenEof = "
            << mHaveSeenEof
            << " mPendingIoRequests = "
            << mPendingIoRequests
            << war_endl;
    }
#endif
    
    if (mHaveSeenEof && !mPendingIoRequests)
        OnTransferDone(WarError());
}

void WarSvrProtocolFtpDataConn::OnTransferDone(const WarError& status)
{
    WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnTransferDone);

    war_datasck_ptr_t my_ptr = this;

    if (mHasFinished)
        return;

    // Kill transfer plugins
    GetPlugins(WSPE_PRE_ON_RECEIVED).erase(
        GetPlugins(WSPE_PRE_ON_RECEIVED).begin(),
        GetPlugins(WSPE_PRE_ON_RECEIVED).end());
    
    GetPlugins(WSPE_PRE_ON_SENT).erase(
        GetPlugins(WSPE_PRE_ON_SENT).begin(),
        GetPlugins(WSPE_PRE_ON_SENT).end());
    
    // Clean up databuffers
    KillBuffers();

    WAR_PERFMON_DEC(WAR_PRFSVR_TRANSFERS_NOW)

    mHasFinished = true;
    mrServer.OnTransferDone(status);
    if (mFilePtr->IsOpen())
        mFilePtr->Close();
    mFilePtr = NULL;

    if (status)
    {
        WAR_PERFMON_INC(WAR_PRFSVR_TRANSFERS_FAILED);
    }
    else
    {
        WAR_PERFMON_INC(WAR_PRFSVR_TRANSFERS_TOTAL);
    }
    
    // Cleanup
    assert(IsOpen() == false);
}

void WarSvrProtocolFtpDataConn::OnAccept(const WarError& status,
        war_socket_t newSocket,
        const WarNetAddress& remoteAddress,
        const WarNetAddress& localAddress)
{
    WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnAccept);

    AUTO_LOCK

    if (status)
    {
        OnTransferDone(status);
        return;
    }

    // We take over the new socket, and call OnConnect()
    mpCompanion->Close();
    mpCompanion->AssignSocket(newSocket);
    SetCurrentState(WAR_SCKSTATE_CONNECTED);
    OnConnect(status);
}

void WarSvrProtocolFtpDataConn::OnConnect(const WarError& status)
{
    WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnConnect);

    WarLog debug_log(WARLOG_DEBUG, "WarSvrProtocolFtpDataConn::OnConnect()");

    AUTO_LOCK

    if (status)
    {
        if (debug_log)
        {
            debug_log << "Data connection with peer failed."
                << status
                << war_endl;
        }
        OnTransferDone(status);
        return;
    }

    if (debug_log)
    {
        debug_log << "Data connection with peer is established."
            << war_endl;
    }

    PumpData();
}

void WarSvrProtocolFtpDataConn::CallbackFunc(
    war_cptr_t ptrToObject, 
    const WarError&, 
    war_transfer_buffer_ptr_t& bufferPtr)
{
    WarSvrProtocolFtpDataConn *pObject = 
        (WarSvrProtocolFtpDataConn *)ptrToObject;

    pObject->OnFileIo(WarError(), bufferPtr);
}

void WarSvrProtocolFtpDataConn::OnFileIo(const WarError& status, 
                                         war_transfer_buffer_ptr_t& bufferPtr)
{
    AUTO_LOCK

    {
        WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo);

        WarError my_status = status;
        WarLog debug_log(WARLOG_DEBUG, "WarSvrProtocolFtpDataConn::OnFileIo()");
        
        {        
            --mPendingIoRequests;
            --mNumBuffers;
            
            if (mHasFinished)
            {
                // Just ignore the event
                if (debug_log)
                {
                    debug_log << "Trashing I/O callback. The connection is already closed. "
                        << GetReferenceCount() << " references left and "
                        << mPendingIoRequests << " pendig IO requests."
                        << war_endl;
                }
                return;
            }
            
            if (D_SENDING == mDirection)
            {
                // Sending to client
                // Pass the buffer to the socket
                if (my_status)
                {
                    WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_ERROR);

                    if (my_status.LocalError() == WAR_FERR_END_OF_FILE)
                    {
                        mHaveSeenEof = true;
                    }
                    else
                    {
                        goto abort_transfer;
                    }
                }
                else
                {
                    if (bufferPtr->mSerialNum == mIoSerialFirst)
                    {
                        WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_SendWithCallback);
                        ++mIoSerialFirst;
                        
                        try
                        {
                            SendWithCallback(bufferPtr, mNumBufSegments);
                            ++mPendingIoRequests;
                            ++mNumBuffers;
                        }
                        catch(WarException& ex)
                        {
                            WarLog warn_log(WARLOG_WARNINGS, "WarSvrProtocolFtpDataConn::OnFileIo()");
                            warn_log << "Caught exception during SendWithCallback() "
                                << ex.Explain()
                                << " Aborting transfer"
                                << war_endl;
                            
                            my_status = ex;
                            goto abort_transfer;
                        }
                    }
                    else
                    {
                        WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_QUEUE);
                        mPendingInQueue.insert(bufferPtr);
                        ++mNumBuffers;
#ifdef _DEBUG
                        if (mPendingInQueue.size() >= mNumBuffers)
                        {
                            debug_log << "Error: Queued set out of sync! \n"
                                << "mIoSerialFirst = "
                                << mIoSerialFirst
                                << "mIoSerialLast="
                                << mIoSerialLast
                                << " mPendingInQueue.size()="
                                << mPendingInQueue.size()
                                << "\n Serials in set: ";
                            
                            for(bufq_set_t::const_iterator P = mPendingInQueue.begin()
                                ; P != mPendingInQueue.end()
                                ; P++)
                            {
                                debug_log << (*P)->mSerialNum << " ";
                            }
                            
                            debug_log << war_endl;
                            WarTime::Sleep(500);
                        }
                        
                        assert(mPendingInQueue.size() < mNumBuffers);
#endif
                    }
                }
            }
            else
            {
                // Receiving from client
                WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnFileIo_RECV);
                if (my_status)
                    goto abort_transfer;
                
                if (!mHaveSeenEof)
                {
                    // Add the buffer to the free-queue
                    mFreeQueue.push_front(bufferPtr);
                    ++mNumBuffers;
                }
            }
        }
        goto done;

abort_transfer:
        {
            // Fatal error. Abort the transfer!
            mFilePtr->Close();
            try
            {
                OnTransferDone(my_status);
            }
            catch(WarException& ex)
            {
                WarLog warn_log(WARLOG_WARNINGS, "WarSvrProtocolFtpDataConn::OnFileIo()");
                warn_log << "Caught exception during OnTransferDone() "
                    << ex.Explain()
                    << " Ignoring"
                    << war_endl;
            }
            return;
        }
    }

done:
    
    PumpData();
}

void WarSvrProtocolFtpDataConn::OnReceived(const WarError& status, 
        war_transfer_buffer_ptr_t& buffer)
{

    AUTO_LOCK
    assert(D_RECEIVING == mDirection);

    {
        WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnReceived);
        
        {   
            --mPendingIoRequests;
            --mNumBuffers;
            
            if (mHasFinished)
            {
                WarLog debug_log(WARLOG_DEBUG, "WarSvrProtocolFtpDataConn::OnReceived()");
                
                // Just ignore the event
                if (debug_log)
                {
                    debug_log << "Trashing I/O callback. The connection is already closed. "
                        << GetReferenceCount() << " references left and "
                        << mPendingIoRequests << " pendig IO requests."
                        << war_endl;
                }
                return;
            }
            
            if (mHaveSeenEof)
                goto seen_eof;
            
            if (status)
            {
                if (status.LocalError() == WAR_FERR_END_OF_FILE)
                {
                    mHaveSeenEof = true;
seen_eof:
                    goto pump_data;
                }
                else
                {
                    // Fatal error. Abort the transfer!
                    mFilePtr->Close();
                    OnTransferDone(status);
                    Close();
                }
            }
            else
            {   
                // Pass buffer to file
                if (mUseFileCallback)
                {
                    mFilePtr->WriteWithCallback(buffer);
                    ++mPendingIoRequests;
                    ++mNumBuffers;
                }
                else
                {
                    try
                    {
                        mFilePtr->Write(buffer->mBuffer,
                            buffer->mBytesUsed);
                    }
                    catch(WarException& ex)
                    {
                        WarLog err_log(WARLOG_ERROR, "WarSvrProtocolFtpDataConn::OnReceived()");
                        
                        err_log << "Failed to write to destination file "
                            << ex.Explain()
                            << " Aborting transfer."
                            << war_endl;
                        
                        OnTransferDone(ex);
                        return;
                    }
                    
                    mFreeQueue.push_front(buffer);
                    ++mNumBuffers;
                    goto pump_data;
                }
            }
        }
    }

    return;

pump_data:
    PumpData();
}
    
void WarSvrProtocolFtpDataConn::OnSent(const WarError& status,
        war_transfer_buffer_ptr_t& buffer)
{
    AUTO_LOCK
    {
        WAR_PROFILE_FUNC(WAR_FP_WarSvrProtocolFtpDataConn_OnSent);
            
        assert(D_SENDING == mDirection);
        
        {
            --mPendingIoRequests;
            --mNumBuffers;
            
            if (mHasFinished)
            {
                WarLog debug_log(WARLOG_DEBUG, "WarSvrProtocolFtpDataConn::OnSent()");
                // Just ignore the event
                if (debug_log)
                {
                    debug_log << "Trashing I/O callback. The connection is already closed. "
                        << GetReferenceCount() << " references left and "
                        << mPendingIoRequests << " pendig IO requests."
                        << war_endl;
                }
                return;
            }
            
            if (status)
            {
                // Fatal error. Abort the transfer!
                mFilePtr->Close();
                OnTransferDone(status);
                return;
            }
            
            if (!mHaveSeenEof)
            {
                mFreeQueue.push_front(buffer);
                ++mNumBuffers;
            }
        }
    }
    
    PumpData();
}

/////////////////////////////// PRIVATE    ///////////////////////////////////

