#include "StdAfx.h"
#include "WarUserAuthDataWin32Nt.h"   // class implemented
#include "WarUtf8.h"
#include "WarSvrPath.h"
#include "WarSvrProperties.h"

#ifndef WAR_INCLUDED_TCHAR_H
#   define WAR_INCLUDED_TCHAR_H
#   include <tchar.h>
#endif

#ifndef WAR_FUNCTION_PROFILER_H
#   include "WarFunctionProfiler.h"
#endif

#ifndef WAR_LOG_H
#   include "WarLog.h"
#endif

/////////////////////////////// PUBLIC ///////////////////////////////////////

//============================= LIFECYCLE ====================================
//============================= OPERATORS ====================================
//============================= OPERATIONS ===================================

void WarUserAuthDataWin32Nt::BeginImpersonate()
throw(WarException)
{
    WAR_PROFILE_FUNC(WAR_FP_WarUserAuthDataWin32Nt_BeginImpersonate);

    if (NULL == mLogonHandle)
        return;

    if (!::ImpersonateLoggedOnUser(mLogonHandle))
    {
        WarLog err_log(WARLOG_ERROR, "WarUserAuthDataWin32Nt::BeginImpersonate()");
        WarSystemError sys_err;

        err_log << "Failed to impersonate user "
            << GetUserName()
            << sys_err
            << war_endl;

        WarThrow(sys_err, NULL);
    }

    ++mImpersonateCnt;
}
    
void WarUserAuthDataWin32Nt::EndImpersonate()
throw(WarException)
{
    WAR_PROFILE_FUNC(WAR_FP_WarUserAuthDataWin32Nt_EndImpersonate);

    if (0 < mImpersonateCnt)
    {
        --mImpersonateCnt;
        ::RevertToSelf();
        assert(0 == mImpersonateCnt);
    }
}


void WarUserAuthDataWin32Nt::MergeProperties(
    WarSvrProperties & effectiveProperties)
{
    // Enumerate the file-exports
    WarWin32Registry reg_file_systems, reg_fsys;
    WarWin32Registry::keyname_list_t file_systems;
    WarWin32Registry::keyname_list_t::const_iterator P;

    try
    {
        reg_file_systems.Open(HKEY_LOCAL_MACHINE, 
            mRegRoot.GetPath(WAR_WINNT_REG_FILE_SYSTEMS),
            KEY_READ);
    }
    catch(WarException& e)
    {
        if (e.LocalError() == WAR_ERR_NAME_DONT_EXIST)
            goto skip_file_systems;
    }
    
    reg_file_systems.EnumKey(file_systems);
    for(P = file_systems.begin()
        ; P != file_systems.end()
        ; P++)
    {
        reg_fsys.Open(HKEY_LOCAL_MACHINE, 
            reg_file_systems.GetPath(P->c_str()), KEY_READ);
        
        DWORD perms = reg_fsys.GetIntValue(_T("Permissions"));
        WarUtf8 mount_point = P->c_str();
        WarUtf8 url = reg_fsys.GetStrValue(_T("Url"));

        WarSvrPath new_path(mount_point.GetUtf8().c_str(), 
            url.GetUtf8().c_str(), perms);

        // Remove the path from the effectiveProperties if it
        // already exist
        WarSvrProperties::path_set_t::iterator P 
            = effectiveProperties.mPaths.find(new_path);
        if (P != effectiveProperties.mPaths.end())
            effectiveProperties.mPaths.erase(P);
        
        effectiveProperties.mPaths.insert(new_path);
    }

skip_file_systems:
    
    return;
}

//============================= ACCESS     ===================================
//============================= INQUIRY    ===================================
/////////////////////////////// PROTECTED  ///////////////////////////////////

/////////////////////////////// PRIVATE    ///////////////////////////////////

