/** 
*/

#ifndef WAR_OPTION_H
#define WAR_OPTION_H

/* SYSTEM INCLUDES */


/* PROJECT INCLUDES */

#ifndef WAR_SMART_POINTER_H
#   include "WarSmartPointer.h"
#endif

#ifndef WAR_COLLECTOR_H
#   include "WarCollector.h"
#endif

#ifndef WAR_AUTO_LOCK_H
#   include "WarAutoLock.h"
#endif

#ifndef WAR_PTR_WRAPPER_H
#   include "WarPtrWrapper.h"
#endif

/* LOCAL INCLUDES */


/* FORWARD REFERENCES */

class WarOptionList;

#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/


/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/

#ifdef __cplusplus

class WarOption : public WarSmartPointer
{
public:
    typedef WarCollector<char> buffer_t;

    enum WarOptionTypeE
    {
        OT_BOOL,
        OT_STRING,
        OT_UNSIGNED,
        OT_INT,
        OT_FLOAT,
        OT_DATE,
        OT_INVALID
    };

    // LIFECYCLE
    
    /**
    * Default constructor.
    */
    WarOption(war_ccstr_t name, 
        war_ccstr_t value = NULL,
        war_ccstr_t defaultValue = NULL,
        war_ccstr_t description = NULL,
        WarOptionTypeE type = OT_STRING)
        throw(WarException)
        : mName(name),
        mValue(value ? value : ""),
        mDefaultValue(defaultValue ? defaultValue : ""),
        mType(type),
        mDescription(description ? description: "")
    {}

    /**
    * Copy constructor.
    *
    * @param from The value to copy to this object.
    */
    WarOption(const WarOption& from)
    {
        operator = (from);
    }
    
    /**
    * Destructor.
    */
    virtual ~WarOption(void) {}
    
    // OPERATORS

    bool operator == (const WarOption& from) const
    {
        WarAutoLock my_lock((WarCriticalSection &)mLock);
        return mName == from.mName;
    }

    bool operator < (const WarOption& from) const
    {
        WarAutoLock my_lock((WarCriticalSection &)mLock);
        return mName < from.mName;
    }

    
    /**
    * Assignment operator.
    *
    * @param from THe value to assign to this object.
    *
    * @return A reference to this object.
    */
    WarOption& operator = (const WarOption& from)
    {
        WarAutoLock my_lock((WarCriticalSection &)mLock);

        mName = from.GetName();
        mValue = from.GetValue();
        mDefaultValue = from.GetDefaultValue();
        mType= from.GetType();
        mDescription = from.GetDescription();

        return *this;
    }
    
    // OPERATIONS  
    
    template <class typeT>
    void SetValue(const typeT& value) 
        throw(WarException)
    {       
        buffer_t buf;
        buf << value;
        DoSetValue(buf);
    }

    template <class typeT>
    void SetDefaultValue(const typeT& value) 
        throw(WarException)
    {
        buffer_t buf;
        buf << value;
        DoSetDefaultValue(buf);
    }

    template <class typeT>
    void SetDescription(const typeT& value)
    {
        buffer_t buf;
        buf << value;
        DoSetDescription(buf);
    }

    // ACCESS

    virtual const std::string GetValue() const
        throw(WarException)
    {
        WarAutoLock my_lock((WarCriticalSection &)mLock);
        return mValue;
    }

    virtual const int GetIntValue() const
        throw(WarException)
    {
        WarAutoLock my_lock((WarCriticalSection &)mLock);
        return mValue.empty() 
            ? 0  
            : atoi(mValue.c_str());
    }
    
    virtual const std::string& GetName() const
        throw(WarException)
    {
        WarAutoLock my_lock((WarCriticalSection &)mLock);
        return mName;
    }
    
    virtual const std::string GetDefaultValue() const
        throw(WarException)
    {
        WarAutoLock my_lock((WarCriticalSection &)mLock);
        return mDefaultValue;
    }

    virtual const int GetDefaultIntValue() const
        throw(WarException)
    {
        WarAutoLock my_lock((WarCriticalSection &)mLock);
        return mDefaultValue.empty() 
            ? 0  
            : atoi(mDefaultValue.c_str());
    }
    
    virtual const std::string GetDescription() const
        throw(WarException)
    {
        WarAutoLock my_lock((WarCriticalSection &)mLock);
        return mDescription;
    }
    
    virtual const WarOptionTypeE GetType() const
        throw(WarException)
    {
        return mType;
    }
    
    // INQUIRY
    
    virtual bool IsEmpty() const
        throw(WarException)
    {
        WarAutoLock my_lock((WarCriticalSection &)mLock);
        return mValue.empty();
    }
    
protected:
    WarCriticalSection mLock;
    
    virtual void DoSetValue(const buffer_t& value)
        throw(WarException)
    {
        WarAutoLock my_lock(mLock); 
        mValue = value.GetValue();
    }

    virtual void DoSetDefaultValue(const buffer_t& value)
        throw(WarException)
    {
        WarAutoLock my_lock(mLock); 
        mDefaultValue = value.GetValue();
    }

    virtual void DoSetDescription(const buffer_t& value)
        throw(WarException)
    {
        WarAutoLock my_lock(mLock); 
        mDescription = value.GetValue();
    }

    virtual void DoSetType(const WarOptionTypeE type)
    {
        WarAutoLock my_lock(mLock);
        mType = type;
    }

private:
    friend WarOptionList;
    std::string mName;
    std::string mValue;
    std::string mDefaultValue;
    WarOptionTypeE mType;
    std::string mDescription;
};

/* INLINE METHODS */

/* EXTERNAL REFERENCES */

typedef WarPtrWrapper<WarOption> war_option_ptr_t;

#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* WAR_OPTION_H_ */

