/** 
*/

#ifndef WAR_SOCKET_PLUGINS_H
#define WAR_SOCKET_PLUGINS_H

/* SYSTEM INCLUDES */

#ifndef WAR_QUEUE_INCLUDED
#   define WAR_QUEUE_INCLUDED
#   include <queue>
#endif

/* PROJECT INCLUDES */

#ifndef WAR_PLUGIN_H
#   include "WarPlugin.h"
#endif

#ifndef WAR_TRANSFER_BUFFER_H
#   include "WarTransferBuffer.h"
#endif

#ifndef WAR_CPS_H
#   include "WarCps.h"
#endif

#ifndef WAR_TIMER_GENERIC_EVENT_H
#   include "WarTimerGenericEvent.h"
#endif

#ifndef WAR_AUTO_COUNTER_H
#   include "WarAutoCounter.h"
#endif

/* LOCAL INCLUDES */


/* FORWARD REFERENCES */

class WarSocket;

#ifdef __cplusplus
extern "C" 
{
#endif
/****************** BEGIN OLD STYLE C spesific ********/

enum WarSocketPluginEventsE
{
    WSPE_PRE_ON_RECEIVED,
    WSPE_PRE_ON_SENT,
    WSPE_INVALID
};

/****************** END OLD STYLE C spesific **********/
#ifdef __cplusplus 
}
#endif

/****************** BEGIN C++ spesific ****************/

#ifdef __cplusplus

class WarSocketPluginPreOnXmit
: public WarPlugin
{
public:
    typedef std::pair<WarError, war_transfer_buffer_ptr_t> data_t;
    typedef std::list<data_t> buffer_queue_t;
    typedef WarTimerGenericEvent<WarSocketPluginPreOnXmit> timer_event_t;
    typedef WarPtrWrapper<timer_event_t> timer_ptr_t;
        
    static WarSocketPluginEventsE GetId()
    {
        return WSPE_PRE_ON_SENT;
    }

    WarSocketPluginPreOnXmit(
        WarSocket *pSocket,
        const unsigned delayMs,
        const unsigned holdMs,
        const unsigned minCnt,
        const unsigned maxCnt,
        const unsigned minCps,
        const unsigned maxCps);

    void OnProcess(const WarError& status,
        war_transfer_buffer_ptr_t& buffer);

    void OnTimer();

protected:
    void SetTimer();
    virtual void OnXmitted(data_t& data) = 0;

    WarCps mCps;
    WarSocket *mpSocket;
    const int mMaxCps;
    buffer_queue_t mBuffers;
    timer_ptr_t mTimerPtr;
    WarCriticalSection mLock;
};


/** Called before WarSocket::OnReceived()
*/
class WarSocketPluginPreOnReceived 
: public WarSocketPluginPreOnXmit
{
public:
    static WarSocketPluginEventsE GetId()
    {
        return WSPE_PRE_ON_RECEIVED;
    }

    WarSocketPluginPreOnReceived(
        WarSocket *pSocket,
        const unsigned delayMs,
        const unsigned holdMs,
        const unsigned minCnt,
        const unsigned maxCnt,
        const unsigned minCps,
        const unsigned maxCps)
        : WarSocketPluginPreOnXmit(
        pSocket,
        delayMs,
        holdMs,
        minCnt,
        maxCnt,
        minCps,
        maxCps)
    {}

protected:
    virtual void OnXmitted(data_t& data);

    ~WarSocketPluginPreOnReceived() {}
};

/** Called before WarSocket::OnReceived()
*/
class WarSocketPluginPreOnSent 
: public WarSocketPluginPreOnXmit
{
public:
    static WarSocketPluginEventsE GetId()
    {
        return WSPE_PRE_ON_SENT;
    }

    WarSocketPluginPreOnSent(
        WarSocket *pSocket,
        const unsigned delayMs,
        const unsigned holdMs,
        const unsigned minCnt,
        const unsigned maxCnt,
        const unsigned minCps,
        const unsigned maxCps)
        : WarSocketPluginPreOnXmit(
        pSocket,
        delayMs,
        holdMs,
        minCnt,
        maxCnt,
        minCps,
        maxCps)
    {}

protected:
    virtual void OnXmitted(data_t& data);

    ~WarSocketPluginPreOnSent() {}
};


/* INLINE METHODS */

/* EXTERNAL REFERENCES */

#endif /* __cplusplus */

/****************** END C++ spesific ******************/

#endif  /* WAR_SOCKET_PLUGINS_H_ */

